@@ -30,17 +30,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-// for some weird reason ltdl.h doesn't compile correctly when not included
-// first
-#if HAVE_FOX_1_0
-# if !defined(WIN32) || defined(__CYGWIN__)
-extern "C" {
-#  include "../ltdl/ltdl.h"
-}
-# endif
-#endif
-
-
 #if !defined(WIN32) || defined(__CYGWIN__)
 # if defined(__CYGWIN__)
 #  include <windows.h>
@@ -49,7 +38,13 @@ extern "C" {
 #  endif
 # endif
 # include <sys/time.h>
-# if HAVE_FOX_1_2
+# if HAVE_FOX_1_4
+#  include <fox-1.4/fx.h>
+#  include <fox-1.4/fxkeys.h>
+# elif HAVE_FOX_1_6
+#  include <fox-1.6/fx.h>
+#  include <fox-1.6/fxkeys.h>
+# elif HAVE_FOX_1_2
 #  include <fox-1.2/fx.h>
 #  include <fox-1.2/fxkeys.h>
 # else
@@ -60,6 +55,7 @@ extern "C" {
 # if defined(__MINGW32__) && defined(PIC) && !defined(FOXDLL)
 #   define FOXDLL
 # endif
+# pragma warning (disable : 4786)
 # include <time.h>
 # include <windows.h>
 # include <fx.h>
@@ -71,11 +67,6 @@ extern "C" {
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 
-#if HAVE_FOX_1_2
-#define getRoot getRootWindow
-#define setTextFont setFont
-#endif
-
 #include <map>
 using namespace std;
 
@@ -92,14 +83,21 @@ Palette::Palette() {
 	used = 0;
 	allowRealization = false;
 	visual = 0;
+	size = 100;
+	entries = new ColourPair[size];
 }
 
 Palette::~Palette() {
 	Release();
+	delete []entries;
+	entries = 0;
 }
 
 void Palette::Release() {
 	used = 0;
+	delete []entries;
+	size = 100;
+	entries = new ColourPair[size];
 }
 
 // This method either adds a colour to the list of wanted colours (want==true)
@@ -112,11 +110,19 @@ void Palette::WantFind(ColourPair &cp, bool want) {
 				return;
 		}
 	
-		if (used < numEntries) {
-			entries[used].desired = cp.desired;
-			entries[used].allocated.Set(cp.desired.AsLong());
-			used++;
+		if (used >= size) {
+			int sizeNew = size * 2;
+			ColourPair *entriesNew = new ColourPair[sizeNew];
+			for (int j=0; j<size; j++) {
+				entriesNew[j] = entries[j];
+			}
+			delete []entries;
+			entries = entriesNew;
+			size = sizeNew;
 		}
+		entries[used].desired = cp.desired;
+		entries[used].allocated.Set(cp.desired.AsLong());
+		used++;
 	} else {
 		for (int i=0; i < used; i++) {
 			if (entries[i].desired == cp.desired) {
@@ -162,6 +168,8 @@ static const char *CharacterSetName(int characterSet) {
 		return "*-*";
 	case SC_CHARSET_RUSSIAN:
 		return "*-r";
+	case SC_CHARSET_CYRILLIC:
+		return "*-cp1251";
 	case SC_CHARSET_SHIFTJIS:
 		return "jisx0208.1983-*";
 	case SC_CHARSET_SYMBOL:
@@ -177,7 +185,9 @@ static const char *CharacterSetName(int characterSet) {
 	case SC_CHARSET_VIETNAMESE:
 		return "*-*";
 	case SC_CHARSET_THAI:
-		return "*-1";
+		return "iso8859-11";
+	case SC_CHARSET_8859_15:
+		return "iso8859-15";
 	default:
 		return "*-*";
 	}
@@ -193,8 +203,13 @@ void Font::Create(const char *faceName, int characterSet,
 	}
 	else {
         id = new FXFont(FXApp::instance(), faceName, size,
+#if HAVE_FOX_1_6
+        	bold ? FXFont::Bold : FXFont::Normal ,
+			italic ? FXFont::Italic : FXFont::Straight,
+#else
         	bold ? FONTWEIGHT_BOLD : FONTWEIGHT_NORMAL,
 			italic ? FONTSLANT_ITALIC : FONTSLANT_REGULAR,
+#endif
 			characterSet);
 	}
 	if (!id) {
@@ -231,6 +246,8 @@ static int CharacterSetCode(int characterSet) {
 		return FONTENCODING_DEFAULT;
 	case SC_CHARSET_RUSSIAN:
 		return FONTENCODING_RUSSIAN;
+	case SC_CHARSET_CYRILLIC:
+		return FONTENCODING_CYRILLIC;
 	case SC_CHARSET_SHIFTJIS:
 		return FONTENCODING_DEFAULT;
 	case SC_CHARSET_SYMBOL:
@@ -244,9 +261,11 @@ static int CharacterSetCode(int characterSet) {
 	case SC_CHARSET_ARABIC:
 		return FONTENCODING_ARABIC;
 	case SC_CHARSET_VIETNAMESE:
-		return FONTENCODING_DEFAULT;
+		return FONTENCODING_CP1258;     /// Windows Vietnam
 	case SC_CHARSET_THAI:
 		return FONTENCODING_THAI;
+	case SC_CHARSET_8859_15:
+		return FONTENCODING_ISO_8859_15;
 	default:
 		return FONTENCODING_DEFAULT;
 	}
@@ -256,9 +275,14 @@ void Font::Create(const char *faceName, int characterSet,
 	int size, bool bold, bool italic, bool) {
 	Release();
 	id = new FXFont(FXApp::instance(), faceName, size,
-									bold ? FONTWEIGHT_BOLD : FONTWEIGHT_NORMAL,
-									italic ? FONTSLANT_ITALIC : FONTSLANT_REGULAR,
-									CharacterSetCode(characterSet));
+#if HAVE_FOX_1_6
+        	bold ? FXFont::Bold : FXFont::Normal ,
+			italic ? FXFont::Italic : FXFont::Straight,
+#else
+        	bold ? FONTWEIGHT_BOLD : FONTWEIGHT_NORMAL,
+			italic ? FONTSLANT_ITALIC : FONTSLANT_REGULAR,
+#endif
+			CharacterSetCode(characterSet));
 	if (!id) {
 		// Font not available so substitute with the app default font.
 		id = FXApp::instance()->getNormalFont();
@@ -311,6 +335,8 @@ public:
 	void FillRectangle(PRectangle rc, ColourAllocated back);
 	void FillRectangle(PRectangle rc, Surface &surfacePattern);
 	void RoundedRectangle(PRectangle rc, ColourAllocated fore, ColourAllocated back);
+	void AlphaRectangle(PRectangle rc, int cornerSize, ColourAllocated fill, int alphaFill,
+		ColourAllocated outline, int alphaOutline, int flags);
 	void Ellipse(PRectangle rc, ColourAllocated fore, ColourAllocated back);
 	void Copy(PRectangle rc, Point from, Surface &surfaceSource);
 
@@ -527,6 +553,64 @@ void SurfaceImpl::RoundedRectangle(PRectangle rc, ColourAllocated fore, ColourAl
 	}
 }
 
+// Plot a point into a guint32 buffer symetrically to all 4 qudrants
+static void AllFour(FXImage *image, int width, int height, int x, int y, FXColor color) {
+	image->setPixel(x, y, color);
+	image->setPixel(width-1-x, y, color);
+	image->setPixel(x, height-1-y, color);
+	image->setPixel(width-1-x, height-1-y, color);
+}
+
+static unsigned int GetRed(unsigned int co) {
+	return (co >> 16) & 0xff;
+}
+
+static unsigned int GetGreen(unsigned int co) {
+	return (co >> 8) & 0xff;
+}
+
+static unsigned int GetBlue(unsigned int co) {
+	return co & 0xff;
+}
+
+void SurfaceImpl::AlphaRectangle(PRectangle rc, int cornerSize, ColourAllocated fill, int alphaFill,
+		ColourAllocated outline, int alphaOutline, int flags) {
+	if (dc()) {
+		int width = rc.Width();
+		int height = rc.Height();
+		// Ensure not distorted too much by corners when small
+		cornerSize = Platform::Minimum(cornerSize, (Platform::Minimum(width, height) / 2) - 2);
+		// Make a 32 bit deep image
+		FXImage * image = new FXImage(FXApp::instance(), NULL, IMAGE_OWNED, width, height);
+
+		FXColor valEmpty = 0;
+		FXColor valFill = FXRGBA(GetRed(fill.AsLong()), GetGreen(fill.AsLong()), GetBlue(fill.AsLong()), alphaFill);
+		FXColor valOutline = FXRGBA(GetRed(outline.AsLong()), GetGreen(outline.AsLong()), GetBlue(outline.AsLong()), alphaOutline);
+		for (int y=0; y<height; y++) {
+			for (int x=0; x<width; x++) {
+				if ((x==0) || (x==width-1) || (y == 0) || (y == height-1)) {
+					image->setPixel(x, y, valOutline);
+				} else {
+					image->setPixel(x, y, valFill);
+				}
+			}
+		}
+		for (int c=0;c<cornerSize; c++) {
+			for (int x=0;x<c+1; x++) {
+				AllFour(image, width, height, x, c-x, valEmpty);
+			}
+		}
+		for (int x=1;x<cornerSize; x++) {
+			AllFour(image, width, height, x, cornerSize-x, valOutline);
+		}
+
+		// Draw with alpha
+		image->create();
+		_dc->drawImage(image, rc.left, rc.top);
+	}
+}
+
+
 void SurfaceImpl::Ellipse(PRectangle rc, ColourAllocated fore, ColourAllocated back) {
 	if (dc()) {
 		PenColour(back);
@@ -553,8 +637,7 @@ void SurfaceImpl::DrawTextBase(PRectangle rc, Font &font_, int ybase, const char
                        ColourAllocated fore) {
 	if (dc()) {
 		PenColour(fore);
-		_dc->setTextFont(font_.GetID());
-
+		_dc->setFont(font_.GetID());
 		const int segmentLength = 1000;
 		int x = rc.left;
 		while ((len > 0) && (x < maxCoordinate)) {
@@ -714,8 +797,8 @@ void Window::SetPositionRelative(PRectangle rc, Window relativeTo) {
 	/* do some corrections to fit into screen */
 	int sizex = rc.right - rc.left;
 	int sizey = rc.bottom - rc.top;
-	int screenWidth = FXApp::instance()->getRoot()->getDefaultWidth();;
-	int screenHeight = FXApp::instance()->getRoot()->getDefaultHeight();;
+	int screenWidth = FXApp::instance()->getRootWindow()->getDefaultWidth();;
+	int screenHeight = FXApp::instance()->getRootWindow()->getDefaultHeight();;
 	if (sizex > screenWidth)
 		ox = 0; /* the best we can do */
 	else if (ox + sizex > screenWidth)
@@ -818,9 +901,10 @@ public:
 	}
 	virtual void Show(bool show=true);
 	virtual void SetFont(Font &font);
-	virtual void Create(Window &parent, int ctrlID, int lineHeight_, bool unicodeMode_);
+	virtual void Create(Window &parent, int ctrlID, Point location_, int lineHeight_, bool unicodeMode_);
 	virtual void SetAverageCharWidth(int width);
 	virtual void SetVisibleRows(int rows);
+	virtual int GetVisibleRows() const;
 	virtual PRectangle GetDesiredRect();
 	virtual int CaretFromEdge();
 	virtual void Clear();
@@ -836,6 +920,7 @@ public:
 		doubleClickAction = action;
 		doubleClickActionData = data;
 	}
+	virtual void SetList(const char* list, char separator, char typesep);
 };
 
 
@@ -885,9 +970,6 @@ PopupListBox::PopupListBox(FXComposite * parent, ListBoxFox * lb) :
 	FXPopup(parent), listBox(lb)
 {
 	list = new FXList(this,
-#if !HAVE_FOX_1_2
-		0,
-#endif
 		this, ID_LIST, LIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y|SCROLLERS_TRACK|HSCROLLER_NEVER);
 	list->setSortFunc(sListSortFunction);
 }
@@ -934,7 +1016,7 @@ long PopupListBox::onDoubleClicked(FXObject *, FXSelector, void *)
 
 // ====================================================================
 
-void ListBoxFox::Create(Window & parent, int, int, bool) {
+void ListBoxFox::Create(Window & parent, int, Point, int, bool) {
 	id = new PopupListBox(static_cast<FXComposite *>(parent.GetID()), this);
 	id->create();
 	list = (static_cast<PopupListBox *>(id))->getList();
@@ -952,6 +1034,10 @@ void ListBoxFox::SetVisibleRows(int rows) {
 	list->setNumVisible(rows);
 }
 
+int ListBoxFox::GetVisibleRows() const {
+	return list->getNumVisible();
+}
+
 PRectangle ListBoxFox::GetDesiredRect() {
 	// Before any size allocated pretend its 100 wide so not scrolled
 	PRectangle rc(0, 0, 100, 100);
@@ -968,7 +1054,7 @@ PRectangle ListBoxFox::GetDesiredRect() {
 			width = 12;
 		rc.right = width * (aveCharWidth+aveCharWidth/3);
 		if (Length() > rows)
-			rc.right += list->verticalScrollbar()->getWidth();
+			rc.right += list->verticalScrollBar()->getWidth();
 
 		// <FIXME/>
 /*		int rows = Length();
@@ -1004,9 +1090,8 @@ PRectangle ListBoxFox::GetDesiredRect() {
 
 void ListBoxFox::Show(bool show) {
 	if (show) {
-		(static_cast<FXPopup *>(id))->popup(NULL,
-																				id->getX(), id->getY(),
-																				id->getWidth(), id->getHeight());
+		(static_cast<FXPopup *>(id))->popup(NULL, id->getX(), id->getY(),
+											id->getWidth(), id->getHeight());
 		list->selectItem(0);
 	}
 }
@@ -1044,10 +1129,12 @@ int ListBoxFox::Length() {
 }
 
 void ListBoxFox::Select(int n) {
+	// Case n==-1 handled by FXList
 	list->setCurrentItem(n, true);
 }
 
 int ListBoxFox::GetSelection() {
+	// Returns -1 when no current item
 	return list->getCurrentItem();
 }
 
@@ -1096,6 +1183,36 @@ void ListBoxFox::ClearRegisteredImages()
 	}	
 }
 
+void ListBoxFox::SetList(const char* list, char separator, char typesep) {
+	Clear();
+	int count = strlen(list) + 1;
+	char *words = new char[count];
+	if (words) {
+		memcpy(words, list, count);
+		char *startword = words;
+		char *numword = NULL;
+		int i = 0;
+		for (; words[i]; i++) {
+			if (words[i] == separator) {
+				words[i] = '\0';
+				if (numword)
+					*numword = '\0';
+				Append(startword, numword?atoi(numword + 1):-1);
+				startword = words + i + 1;
+				numword = NULL;
+			} else if (words[i] == typesep) {
+				numword = words + i;
+			}
+		}
+		if (startword) {
+			if (numword)
+				*numword = '\0';
+			Append(startword, numword?atoi(numword + 1):-1);
+		}
+		delete []words;
+	}
+}
+
 // ====================================================================
 // ListBox
 // ====================================================================
@@ -1132,8 +1249,8 @@ void Menu::Destroy() {
 }
 
 void Menu::Show(Point pt, Window &) {
-	int screenHeight = FXApp::instance()->getRoot()->getDefaultHeight();
-	int screenWidth = FXApp::instance()->getRoot()->getDefaultWidth();
+	int screenHeight = FXApp::instance()->getRootWindow()->getDefaultHeight();
+	int screenWidth = FXApp::instance()->getRootWindow()->getDefaultWidth();
 	id->create();
 	if ((pt.x + id->getWidth()) > screenWidth) {
 		pt.x = screenWidth - id->getWidth();
@@ -1220,24 +1337,23 @@ double ElapsedTime::Duration(bool reset) {
 #endif	// WIN32
 
 // ====================================================================
-// Dynamic library handling. Three ways:
-// - fxdllXxx API with Fox 1.2
-// - libltdl with Fox 1.0 under autotools platforms
-// - Win32 API with Fox 1.0 and Win32 platforms without autotools (VC++)
+// Dynamic library handling.
+// - fxdllXxx API with Fox >= 1.2
 // ====================================================================
 
-// Fox 1.2 has dynamic librarie handling
-#if HAVE_FOX_1_2
+// Fox >= 1.2 has dynamic librarie handling
 
-# if !defined(WIN32) || defined(__CYGWIN__) || defined(__MINGW32__)
-#  if HAVE_FOX_1_2
-#   include <fox-1.2/FXDLL.h>
-#  else
-#   include <fox/FXDLL.h>
-#  endif
-# else
-#  include <FXDLL.h>
-# endif	// !defined(WIN32) || defined(__CYGWIN__) || defined(__MINGW32__)
+#if !defined(WIN32) || defined(__CYGWIN__) || defined(__MINGW32__)
+# if HAVE_FOX_1_2
+#  include <fox-1.2/FXDLL.h>
+# elif HAVE_FOX_1_4
+#  include <fox-1.4/FXDLL.h>
+# elif HAVE_FOX_1_6
+#  include <fox-1.6/FXDLL.h>
+# endif
+#else
+# include <FXDLL.h>
+#endif	// !defined(WIN32) || defined(__CYGWIN__) || defined(__MINGW32__)
 
 class DynamicLibraryImpl : public DynamicLibrary {
 protected:
@@ -1269,76 +1385,6 @@ DynamicLibrary *DynamicLibrary::Load(const char *modulePath) {
 	return static_cast<DynamicLibrary *>( new DynamicLibraryImpl(modulePath) );
 }
 
-// For Fox 1.0, I'd like to use libltdl for both Unix and Win32 but I haven't
-// succeded yet using it under Win32
-#elif !defined(WIN32) || defined(__CYGWIN__)
-
-class DynamicLibraryImpl : public DynamicLibrary {
-protected:
-	lt_dlhandle m;
-public:
-	DynamicLibraryImpl(const char *modulePath) {
-		lt_dlinit();
-		m = lt_dlopen(modulePath);
-	}
-
-	virtual ~DynamicLibraryImpl() {
-		if (m != NULL)
-			lt_dlclose(m);
-		lt_dlexit();
-	}
-
-	// Use lt_dlsym to get a pointer to the relevant function.
-	virtual Function FindFunction(const char *name) {
-		if (m != NULL) {
-			return lt_dlsym(m, name);
-		} else
-			return NULL;
-	}
-
-	virtual bool IsValid() {
-		return m != NULL;
-	}
-};
-
-DynamicLibrary *DynamicLibrary::Load(const char *modulePath) {
-	return static_cast<DynamicLibrary *>( new DynamicLibraryImpl(modulePath) );
-}
-
-#else // !defined(WIN32) || defined(__CYGWIN__)
-
-class DynamicLibraryImpl : public DynamicLibrary {
-protected:
-	HMODULE h;
-public:
-	DynamicLibraryImpl(const char *modulePath) {
-		h = ::LoadLibrary(modulePath);
-	}
-
-	virtual ~DynamicLibraryImpl() {
-		if (h != NULL)
-			::FreeLibrary(h);
-	}
-
-	// Use GetProcAddress to get a pointer to the relevant function.
-	virtual Function FindFunction(const char *name) {
-		if (h != NULL) {
-			return static_cast<Function>(
-				(void *)(::GetProcAddress(h, name)));
-		} else
-			return NULL;
-	}
-
-	virtual bool IsValid() {
-		return h != NULL;
-	}
-};
-
-DynamicLibrary *DynamicLibrary::Load(const char *modulePath) {
-	return static_cast<DynamicLibrary *>( new DynamicLibraryImpl(modulePath) );
-}
-
-#endif // !defined(WIN32) || defined(__CYGWIN__)
 
 // ====================================================================
 // Platform
